#include "firewall.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_firewall_setting_load_
  void firewall_setting_load(const f_console_arguments_t arguments, firewall_main_t * const main) {

    if (!main) return;

    main->setting.state.step_small = firewall_allocation_console_d;

    f_console_parameter_process(arguments, &main->program.parameters, &main->setting.state, 0);

    main->setting.state.step_small = firewall_allocation_small_d;

    if (F_status_is_error(main->setting.state.status)) {
      firewall_print_error(&main->program.error, macro_firewall_f(f_console_parameter_process));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_context_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      firewall_print_error(&main->program.error, macro_firewall_f(fll_program_parameter_process_context_standard));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_verbosity_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      firewall_print_error(&main->program.error, macro_firewall_f(fll_program_parameter_process_verbosity_standard));

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_help_e].result & f_console_result_found_e) {
      main->setting.flag |= firewall_main_flag_help_d;

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_version_e].result & f_console_result_found_e) {
      main->setting.flag |= firewall_main_flag_version_d;

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_copyright_e].result & f_console_result_found_e) {
      main->setting.flag |= firewall_main_flag_copyright_d;

      return;
    }

    f_number_unsigned_t index = 0;

    if (main->program.parameters.array[firewall_parameter_operation_start_e].result & f_console_result_found_e) {
      main->setting.flag &= ~firewall_main_flag_operation_d;
      main->setting.flag |= firewall_main_flag_operation_start_d;

      index = main->program.parameters.array[firewall_parameter_operation_start_e].location;
    }

    if (main->program.parameters.array[firewall_parameter_operation_stop_e].result & f_console_result_found_e) {
      if (!(main->setting.flag & firewall_main_flag_operation_d) || main->program.parameters.array[firewall_parameter_operation_stop_e].location > index) {
        main->setting.flag &= ~firewall_main_flag_operation_d;
        main->setting.flag |= firewall_main_flag_operation_stop_d;

        index = main->program.parameters.array[firewall_parameter_operation_stop_e].location;
      }
    }

    if (main->program.parameters.array[firewall_parameter_operation_restart_e].result & f_console_result_found_e) {
      if (!(main->setting.flag & firewall_main_flag_operation_d) || main->program.parameters.array[firewall_parameter_operation_restart_e].location > index) {
        main->setting.flag &= ~firewall_main_flag_operation_d;
        main->setting.flag |= firewall_main_flag_operation_restart_d;

        index = main->program.parameters.array[firewall_parameter_operation_restart_e].location;
      }
    }

    if (main->program.parameters.array[firewall_parameter_operation_lock_e].result & f_console_result_found_e) {
      if (!(main->setting.flag & firewall_main_flag_operation_d) || main->program.parameters.array[firewall_parameter_operation_lock_e].location > index) {
        main->setting.flag &= ~firewall_main_flag_operation_d;
        main->setting.flag |= firewall_main_flag_operation_lock_d;

        index = main->program.parameters.array[firewall_parameter_operation_lock_e].location;
      }
    }

    if (main->program.parameters.array[firewall_parameter_operation_show_e].result & f_console_result_found_e) {
      if (!(main->setting.flag & firewall_main_flag_operation_d) || main->program.parameters.array[firewall_parameter_operation_show_e].location > index) {
        main->setting.flag &= ~firewall_main_flag_operation_d;
        main->setting.flag |= firewall_main_flag_operation_show_d;
      }
    }

    if (main->program.parameters.array[firewall_parameter_4_e].result & f_console_result_found_e) {
      if (main->program.parameters.array[firewall_parameter_6_e].result & f_console_result_found_e) {
        main->setting.flag |= firewall_main_flag_ipv46_d;
      }
      else {
        main->setting.flag &= ~firewall_main_flag_ipv6_d;
        main->setting.flag |= firewall_main_flag_ipv4_d;
      }
    }
    else if (main->program.parameters.array[firewall_parameter_6_e].result & f_console_result_found_e) {
      main->setting.flag &= ~firewall_main_flag_ipv4_d;
      main->setting.flag |= firewall_main_flag_ipv6_d;
    }

    if (main->program.pipe & fll_program_data_pipe_input_e) {
      main->setting.flag |= firewall_main_flag_pipe_d;
    }
    else {
      main->setting.flag &= ~firewall_main_flag_pipe_d;
    }

    if (main->setting.flag & firewall_main_flag_operation_show_d) {
      if (main->program.parameters.remaining.used) {
        main->setting.flag &= ~firewall_main_flag_operation_show_filter_nat_mangle_d;

        for (f_number_unsigned_t i = 0; i < main->program.parameters.remaining.used; ++i) {

          if (firewall_signal_check(main)) return;

          index = main->program.parameters.remaining.array[i];

          if (f_compare_dynamic(firewall_show_nat_s, main->program.parameters.arguments.array[index]) == F_equal_to) {
            main->setting.flag |= firewall_main_flag_operation_show_nat_d;
          }
          else if (f_compare_dynamic(firewall_show_mangle_s, main->program.parameters.arguments.array[index]) == F_equal_to) {
            main->setting.flag |= firewall_main_flag_operation_show_mangle_d;
          }
          else if (f_compare_dynamic(firewall_show_filter_s, main->program.parameters.arguments.array[index]) == F_equal_to) {
            main->setting.flag |= firewall_main_flag_operation_show_filter_d;
          }
          else {
            firewall_print_warning_show_option_unknown(&main->program.warning, main->program.parameters.arguments.array[index]);
          }
        } // for
      }
      else {
        main->setting.flag |= firewall_main_flag_operation_show_filter_nat_mangle_d;
      }
    }
  }
#endif // _di_firewall_setting_load_

#ifdef __cplusplus
} // extern "C"
#endif
