/**
 * FLL - Level 3
 *
 * Project: Firewall
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common define types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _firewall_common_define_h
#define _firewall_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * firewall_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_firewall_allocation_d_
  #define firewall_allocation_console_d 4
  #define firewall_allocation_large_d   256
  #define firewall_allocation_pipe_d    16384
  #define firewall_allocation_small_d   16
#endif // _di_firewall_allocation_d_

/**
 * Flags representing the current state of the processed rule.
 *
 * firewall_data_is_*_d:
 *   - none:           No flags set.
 *   - global:         The current processed rule is global.
 *   - local:          The current processed rule is lock.
 *   - main:           The current processed rule is main.
 *   - stop:           The current processed rule is stop.
 *   - stop_main_lock: Helper flag representing main, stop, and lock being set.
 */
#ifndef _di_firewall_data_is_d_
  #define firewall_data_is_none_d           0x0
  #define firewall_data_is_global_d         0x1
  #define firewall_data_is_lock_d           0x2
  #define firewall_data_is_main_d           0x4
  #define firewall_data_is_stop_d           0x8
  #define firewall_data_is_stop_main_lock_d 0xe
#endif // _di_firewall_data_is_d_

/**
 * Flags passed to the main function or program.
 *
 * firewall_main_flag_*_d:
 *   - none:                             No flags set.
 *   - copyright:                        Print copyright.
 *   - help:                             Print help.
 *   - ipv4:                             Operate in IPv4 by default or enable ipv4 when calling "show" commands.
 *   - ipv6:                             Operate in IPv6 by default or enable ipv6 when calling "show" commands.
 *   - ipv46:                            A helper flag representing both ipv4 and ipv6 flag bits being set.
 *   - operation:                        A helper flag representing every operation flag bit being set.
 *   - operation_lock:                   Perform the lock operation.
 *   - operation_restart:                Perform the restart operation.
 *   - operation_show:                   Perform the show operation.
 *   - operation_show_nat:               For the show operation, show nat.
 *   - operation_show_mangle:            For the show operation, show mangle.
 *   - operation_show_filter:            For the show operation, show filter.
 *   - operation_show_filter_nat_mangle: A helper flag representing the show nat, show mangle, and short filter flags set.
 *   - operation_start:                  Perform the start operation.
 *   - operation_start_restart:          A helper flag representing both the start and restart operation flag set.
 *   - operation_stop:                   Perform the stop operation.
 *   - operation_stop_restart:           A helper flag representing both the stop and restart operation flag set.
 *   - operation_stop_restart_lock:      A helper flag representing both the stop, restart, and lock operation flag set.
 *   - pipe:                             Use the input pipe.
 *   - version:                          Print version.
 *   - version_copyright_help:           A helper flag representing version, copyright, and help flag bits being set.
 */
#ifndef _di_firewall_main_flag_d_
  #define firewall_main_flag_none_d                             0x0
  #define firewall_main_flag_copyright_d                        0x1
  #define firewall_main_flag_help_d                             0x2
  #define firewall_main_flag_ipv4_d                             0x4
  #define firewall_main_flag_ipv6_d                             0x8
  #define firewall_main_flag_ipv46_d                            0xc
  #define firewall_main_flag_operation_d                        0xff0
  #define firewall_main_flag_operation_lock_d                   0x10
  #define firewall_main_flag_operation_restart_d                0x20
  #define firewall_main_flag_operation_show_d                   0x40
  #define firewall_main_flag_operation_show_nat_d               0x80
  #define firewall_main_flag_operation_show_mangle_d            0x100
  #define firewall_main_flag_operation_show_filter_d            0x200
  #define firewall_main_flag_operation_show_filter_nat_mangle_d 0x380
  #define firewall_main_flag_operation_start_d                  0x400
  #define firewall_main_flag_operation_start_restart_d          0x420
  #define firewall_main_flag_operation_stop_d                   0x800
  #define firewall_main_flag_operation_stop_restart_d           0x820
  #define firewall_main_flag_operation_stop_restart_lock_d      0x830
  #define firewall_main_flag_pipe_d                             0x1000
  #define firewall_main_flag_version_d                          0x2000
  #define firewall_main_flag_version_copyright_help_d           0x2003
#endif // _di_firewall_main_flag_d_

/**
 * The program signal defines.
 *
 * firewall_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_firewall_signal_d_
  #define firewall_signal_check_d          500000
  #define firewall_signal_check_failsafe_d 20000
  #define firewall_signal_check_tiny_d     4
  #define firewall_signal_check_short_d    16
#endif // _di_firewall_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _firewall_common_define_h
